/*
 * $Id: Body.as,v 1.3 2007/09/06 12:35:41 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import org.cove.ape.Group;
  import org.cove.ape.CircleParticle;
  import abagames.util.Vector;

  /**
   * Player's head, hand and foot.
   */
  public class Body extends ApeParticleActor {
    public static const POS_HISTORY_MAX:int = 5;
    private var buffer:BitmapData;
    private var field:Field;
    private var shape:Shape;
    private var shadowRect:Rectangle;
    private var cords:CordPool;
    private var stayCnt:int;
    private var posHistory:Array;
    private var posHistoryIdx:int;

    public function Body(buffer:BitmapData, field:Field, cords:CordPool, group:Group) {
      this.buffer = buffer;
      this.field = field;
      this.cords = cords;
      posHistory = new Array;
      for (var i:int = 0; i < POS_HISTORY_MAX; i++)
        posHistory.push(new Vector);
      clearPosHistory();
      setGroup(group);
    }

    public function clearPosHistory():void {
      for (var i:int = 0; i < POS_HISTORY_MAX; i++)
        posHistory[i].y = 9999;
    }

    public function add(x:Number, y:Number, sz:Number):void {
      shape = new Shape(buffer, field);
      shape.addBox(0, 0, sz, sz, 0, 255, 0);
      _particle = new CircleParticle(0, 0, sz);
      _particle.mass = 0.15;
      _particle.friction = 0.1;
      setPos(x, y);
      posHistoryIdx = 0;
      addToGroup();
      _particle.visible = false;
    }

    public override function update():void {
      if (_particle.px < field.upperLeftPos.x)
        _particle.px = field.upperLeftPos.x;
      else if (_particle.px > field.lowerRightPos.x)
        _particle.px = field.lowerRightPos.x;
      super.update();
      shape.drawDistorted(_pos);
      var ph:Vector = Vector(posHistory[posHistoryIdx]);
      ph.x = _pos.x;
      ph.y = _pos.y;
      posHistoryIdx++;
      if (posHistoryIdx >= POS_HISTORY_MAX)
        posHistoryIdx = 0;
    }

    public function getHistoryPos(idx:int):Vector {
      var phi:int = posHistoryIdx - idx;
      while (phi < 0)
        phi += POS_HISTORY_MAX;
      return posHistory[phi];
    }

    public function drawShape(p:Vector):void {
      shape.drawDistortedWithoutScroll(p);
    }

    public function drawShapeBlur(p:Vector, t:Number):void {
      shape.drawBlur(p, t);
    }

    public override function remove():void {
      cords.removeConnected(this);
      removeFromGroup();
      super.remove();
    }
  }
}
