/*
 * $Id: Field.as,v 1.3 2007/09/06 12:35:42 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import org.cove.ape.Group;
  import org.cove.ape.RectangleParticle;
  import abagames.util.Vector;

  /**
   * Vertical room field.
   */
  public class Field {
    public static const GRAVITY:Number = 3;
    public static const PIXEL_METER_RATIO:Number = 24;
    public var scrollVel:Number;
    private var _scrollY:Number;
    private var buffer:BitmapData;
    private var _size:Vector;
    private var _upperLeftPos:Vector;
    private var _lowerRightPos:Vector;
    private var walls:Array;
    private var rect:Rectangle;
    private var isTitle:Boolean;

    public function Field(buffer:BitmapData, terrain:Group) {
      this.buffer = buffer;
      _size = new Vector(Lbr.SCREEN_WIDTH - 30, Lbr.SCREEN_HEIGHT);
      _upperLeftPos = new Vector(15, 0);
      _lowerRightPos = new Vector(_upperLeftPos.x + _size.x, _upperLeftPos.y + _size.y);
      walls = new Array;
      for (var i:int = 0; i < 2; i++)
        walls.push(null);
      rect = new Rectangle(0, 0, 4, 4);
      var rp:RectangleParticle;
      rp = new RectangleParticle(-40, Lbr.SCREEN_HEIGHT / 2, 100, Lbr.SCREEN_HEIGHT, 0, true);
      rp.setStyle(0, 0, 0, 0xcccccc);
      terrain.addParticle(rp);
      walls[0] = rp;
      rp = new RectangleParticle(Lbr.SCREEN_WIDTH + 40, Lbr.SCREEN_HEIGHT / 2, 100, Lbr.SCREEN_HEIGHT, 0, true);
      rp.setStyle(0, 0, 0, 0xcccccc);
      terrain.addParticle(rp);
      walls[1] = rp;
    }

    public function init():void {
      scrollVel = 0;
      _scrollY = 0;
      updateScroll();
      isTitle = false;
    }

    public function initTitle():void {
      init();
      isTitle = true;
    }

    public function update():void {
      _scrollY += scrollVel;
      updateScroll();
      if (isTitle)
        return;
      var hy:Number = (_scrollY + _size.y) % (PIXEL_METER_RATIO * 10);
      var hn:int = (_scrollY + _size.y) / (PIXEL_METER_RATIO * 10);
      for (var y:Number = hy; y < _size.y + 20; y += (PIXEL_METER_RATIO * 10)) {
        drawLine(y);
        Letter.drawNumLeft(hn * 10, 5, y - 12, 0.7);
        hn--;
      }
    }

    private function updateScroll():void {
      _upperLeftPos.y = -_scrollY;
      _lowerRightPos.y = _upperLeftPos.y + _size.y;
      for (var i:int = 0; i < 2; i++)
        walls[i].py = Lbr.SCREEN_HEIGHT / 2 - _scrollY;
    }

    private function drawLine(y:Number):void {
      for (var x:int = _upperLeftPos.x; x <= _lowerRightPos.x; x += 10) {
        rect.x = x - 2;
        rect.y = y - 2;
        buffer.fillRect(rect, 0x666666);
      }
    }
 
    public function isIn(pos:Vector):Boolean {
      return (pos.x > _upperLeftPos.x && pos.x < _lowerRightPos.x &&
              pos.y > _upperLeftPos.y && pos.y < _lowerRightPos.y);
    }

    public function isInY(y:Number):Boolean {
      return (y > _upperLeftPos.y - _size.y * 0.5 && y < _lowerRightPos.y + _size.y * 0.4);
    }

    public function isInYUpper(y:Number):Boolean {
      return (y > _upperLeftPos.y - _size.y * 0.5 && y < _lowerRightPos.y + _size.y * 0.05);
    }

    public function isInYScreen(y:Number):Boolean {
      return (y > _upperLeftPos.y && y < _lowerRightPos.y);
    }

    public function get size():Vector {
      return _size;
    }

    public function get upperLeftPos():Vector {
      return _upperLeftPos;
    }

    public function get lowerRightPos():Vector {
      return _lowerRightPos;
    }

    public function get scrollY():Number {
      return _scrollY;
    }
  }
}
