/*
 * $Id: Letter.as,v 1.1.1.1 2007/08/19 01:14:45 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import abagames.util.Vector;

  /**
   * Drawing letters.
   */
  public class Letter {
    private static var shapes:Array;
    private static var pos:Vector;

    public static function init(buffer:BitmapData, field:Field):void {
      shapes = new Array;
      var s:Shape;
      s = new Shape(buffer, field);
      s.addBox(0, 0, 10, 2, 220, 220, 220);
      shapes.push(s);
      s = new Shape(buffer, field);
      s.addBox(0, 0, 2, 10, 220, 220, 220);
      shapes.push(s);
      s = new Shape(buffer, field);
      s.addBox(0, 0, 2, 2, 220, 220, 220);
      s.addBox(3, -4, 2, 2, 220, 220, 220);
      s.addBox(-3, 4, 2, 2, 220, 220, 220);
      shapes.push(s);
      s = new Shape(buffer, field);
      s.addBox(0, 0, 2, 2, 220, 220, 220);
      s.addBox(-3, -4, 2, 2, 220, 220, 220);
      s.addBox(3, 4, 2, 2, 220, 220, 220);
      shapes.push(s);
      pos = new Vector;
    }

    public static function drawString(str:String, x:Number, y:Number, s:Number = 1):void {
      var sx:Number = x;
      for (var i:int = 0; i < str.length; i++) {
        var cc:int = int(str.charCodeAt(i));
        if (cc != 0x20)
          draw(convertCharToInt(cc), sx, y, s);
        sx += 20 * s;
      }
    }

    public static function drawNum(num:int, x:Number, y:Number, s:Number = 1):void {
      var sx:Number = x;
      var n:int = num;
      for (;;) {
        var ln:int = n % 10;
        draw(ln, sx, y, s);
        sx -= 20 * s;
        n /= 10;
        if (n <= 0)
          return;
      }
    }

    public static function drawNumLeft(num:int, x:Number, y:Number, s:Number = 1):void {
      var sx:Number = x;
      var n:int = num;
      for (;;) {
        sx += 20 * s;
        n /= 10;
        if (n <= 0)
          break;
      }
      n = num;
      for (;;) {
        var ln:int = n % 10;
        draw(ln, sx, y, s);
        sx -= 20 * s;
        n /= 10;
        if (n <= 0)
          return;
      }
    }

    public static function draw(idx:int, x:Number, y:Number, s:Number = 1):void {
      var pt:Array = patterns[idx];
      var idx:int = 0;
      while (pt[idx][4] != 99999) {
        pos.x = pt[idx][0] * 12 * s + x;
        pos.y = -pt[idx][1] * 11 * s + y;
        if (pt[idx][4] >= 75 && pt[idx][4] <= 105)
          shapes[1].drawWithoutScroll(pos);
        else if (pt[idx][4] == 120 || pt[idx][4] == 360 - 60)
          shapes[2].drawWithoutScroll(pos);
        else if (pt[idx][4] == 60 || pt[idx][4] == 360 - 120)
          shapes[3].drawWithoutScroll(pos);
        else
          shapes[0].drawWithoutScroll(pos);
        idx++;
      }
    }

    private static function convertCharToInt(c:int):int {
      var idx:int;
      if (c >= 0x30 && c <= 0x39)
        idx = c - 0x30;
      else if (c >= 0x41 && c <= 0x5a)
        idx = c - 0x41 + 10;
      else if (c >= 0x61 && c <= 0x7a)
        idx = c - 0x61 + 10;
      else if (c == 0x2e)
        idx = 36;
      else if (c == 0x2d)
        idx = 38;
      else if (c == 0x2b)
        idx = 39;
      else if (c == 0x5f)
        idx = 37;
      else if (c == 0x21)
        idx = 42;
      else if (c == 0x2f)
        idx = 43;
      return idx;
    }

    private static var patterns:Array =
    [[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.6, 0.55, 0.65, 0.3, 90], [0.6, 0.55, 0.65, 0.3, 90],
     [-0.6, -0.55, 0.65, 0.3, 90], [0.6, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [0.5, 0.55, 0.65, 0.3, 90],
     [0.5, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [0.65, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [0.65, 0.55, 0.65, 0.3, 90],
     [0.65, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[//A
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.18, 1.15, 0.45, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.45, 0.55, 0.65, 0.3, 90],
     [-0.18, 0, 0.45, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90],
     [-0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.15, 1.15, 0.45, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.45, 0.45, 0.65, 0.3, 90],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[//F
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90],
     [0.05, 0, 0.3, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 0.55, 0.65, 0.3, 90],
     [0, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [0.65, 0.55, 0.65, 0.3, 90],
     [0.65, -0.55, 0.65, 0.3, 90], [-0.7, -0.7, 0.3, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[//K
     [-0.65, 0.55, 0.65, 0.3, 90], [0.4, 0.55, 0.65, 0.3, 100],
     [-0.25, 0, 0.45, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.6, -0.55, 0.65, 0.3, 80],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.65, 0.55, 0.65, 0.3, 90],
     [-0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.5, 1.15, 0.3, 0.3, 0], [0.1, 1.15, 0.3, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, 0.55, 0.65, 0.3, 90],
     [0, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[//P
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0.05, -0.55, 0.45, 0.3, 60],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [-0.2, 0, 0.45, 0.3, 0],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.45, -0.55, 0.65, 0.3, 80],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [-0.65, 0.55, 0.65, 0.3, 90],
     [0, 0, 0.65, 0.3, 0],
     [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.5, 1.15, 0.55, 0.3, 0], [0.5, 1.15, 0.55, 0.3, 0],
     [0.1, 0.55, 0.65, 0.3, 90],
     [0.1, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[//U
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [-0.5, -0.55, 0.65, 0.3, 90], [0.5, -0.55, 0.65, 0.3, 90],
     [-0.1, -1.15, 0.45, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.65, 0.55, 0.65, 0.3, 90], [0.65, 0.55, 0.65, 0.3, 90],
     [-0.65, -0.55, 0.65, 0.3, 90], [0.65, -0.55, 0.65, 0.3, 90],
     [-0.5, -1.15, 0.3, 0.3, 0], [0.1, -1.15, 0.3, 0.3, 0],
     [0, 0.55, 0.65, 0.3, 90],
     [0, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.4, 0.6, 0.85, 0.3, 360-120],
     [0.4, 0.6, 0.85, 0.3, 360-60],
     [-0.4, -0.6, 0.85, 0.3, 360-240],
     [0.4, -0.6, 0.85, 0.3, 360-300],
     [0, 0, 0, 0, 99999],
    ],[
     [-0.4, 0.6, 0.85, 0.3, 360-120],
     [0.4, 0.6, 0.85, 0.3, 360-60],
     [-0.1, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[
     [0, 1.15, 0.65, 0.3, 0],
     [0.3, 0.4, 0.65, 0.3, 120],
     [-0.3, -0.4, 0.65, 0.3, 120],
     [0, -1.15, 0.65, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[//.
     [0, -1.15, 0.3, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[//_
     [0, -1.15, 0.8, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[//-
     [0, 0, 0.9, 0.3, 0],
     [0, 0, 0, 0, 99999],
    ],[//+
     [-0.5, 0, 0.45, 0.3, 0], [0.45, 0, 0.45, 0.3, 0],
     [0.1, 0.55, 0.65, 0.3, 90],
     [0.1, -0.55, 0.65, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[//'
     [0, 1.0, 0.4, 0.2, 90],
     [0, 0, 0, 0, 99999],
    ],[//''
     [-0.19, 1.0, 0.4, 0.2, 90],
     [0.2, 1.0, 0.4, 0.2, 90],
     [0, 0, 0, 0, 99999],
    ],[//!
     [0.56, 0.25, 1.1, 0.3, 90],
     [0, -1.0, 0.3, 0.3, 90],
     [0, 0, 0, 0, 99999],
    ],[// /
     [0.8, 0, 1.75, 0.3, 120],
     [0, 0, 0, 0, 99999],
    ]];
  }
}
