/*
 * $Id: Player.as,v 1.5 2007/09/06 12:35:42 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import flash.media.SoundChannel;
  import flash.media.SoundTransform;
  import org.cove.ape.Group;
  import org.cove.ape.CircleParticle;
  import abagames.util.Vector;
  import abagames.util.KeyState;

  /**
   * Player consists of body parts and cords.
   */
  public class Player {
    private static const MOVEMENT_FORCE:Number = 32;
    private static const RESTART_CNT:int = 60;
    private static const FIRE_INTERVAL:int = 3;
    private static const FIRE_FORCE:Number = 320;
    private static const FIRE_RECOIL_FORCE:Number = 36;
    private static const MAX_ENERGY:Number = 10000;
    private static const MIN_SCROLL_VELOCITY_Y:Number = 5;
    private static const SCROLL_VELOCITY_Y_RATIO:Number = 0.15;
    private var buffer:BitmapData;
    private var field:Field;
    private var sound:SoundHandler;
    private var mainLoop:MainLoop;
    private var fragments:FragmentPool;
    private var particles:ParticlePool;
    private var stage:Stage;
    private var _pos:Vector;
    private var keyState:KeyState;
    private var restartCnt:int;
    private var shape:Shape;
    private var parts:BodyPool;
    private var cords:CordPool;
    private var fireCnt:int;
    private var fireIdx:int;
    private var fireDeg:Number;
    private var _height:int;
    private var energy:Number;
    private var extendEnergy:Number;
    private var left:int;
    private var energyWarningCnt:int;
    private var rect:Rectangle;
    private var shapePos:Vector;
    private var gameoverCnt:int;
    private var aPressed:Boolean;
    private var isTitle:Boolean;
    private var noBounceCnt:int;

    public function Player(buffer:BitmapData, keyState:KeyState, field:Field,
                         fragments:FragmentPool, particles:ParticlePool,
                         group:Group, sound:SoundHandler, mainLoop:MainLoop) {
      this.buffer = buffer;
      this.keyState = keyState;
      this.field = field;
      this.sound = sound;
      this.mainLoop = mainLoop;
      this.fragments = fragments;
      this.particles = particles;
      _pos = new Vector;
      cords = new CordPool(5, buffer, field, group);
      parts = new BodyPool(6, buffer, field, cords, group);
      rect = new Rectangle(0, 0, 5, 5);
      shapePos = new Vector;
      setBodies();
    }

    public function setParams(stage:Stage):void {
      this.stage = stage;
    }

    private function setBodies():void {
      for (var i:int = 0; i < 6; i++) {
        var b:Body = Body(parts.getInstance());
        if (i == 0)
          b.add(0, 0, 8);
        else
          b.add(0, 0, 5);
      }
      connectBall(1, 0, 5);
      connectBall(1, 2, 10);
      connectBall(1, 3, 10);
      connectBall(1, 4, 15);
      connectBall(1, 5, 15);
    }

    private function connectBall(i1:int, i2:int, l:Number):void {
      var c:Cord = Cord(cords.getInstance());
      c.add(Body(parts.actors[i1]), Body(parts.actors[i2]), l, 0.4, 0, 0xff, 0xff);
    }

    public function init():void {
      left = 2;
      extendEnergy = 0;
      _height = 0;
      respawn();
      gameoverCnt = 0;
      aPressed = true;
      isTitle = false;
    }

    public function initTitle():void {
      respawn();
      setHidden();
      gameoverCnt = 0;
      field.scrollVel = MIN_SCROLL_VELOCITY_Y * SCROLL_VELOCITY_Y_RATIO;
      isTitle = true;
    }

    public function respawn():void {
      _pos.x = field.upperLeftPos.x + field.size.x * 0.5;
      _pos.y = field.lowerRightPos.y;
      Body(parts.actors[0]).setPos(_pos.x, _pos.y - 2);
      Body(parts.actors[1]).setPos(_pos.x, _pos.y);
      Body(parts.actors[2]).setPos(_pos.x - 3, _pos.y);
      Body(parts.actors[3]).setPos(_pos.x + 3, _pos.y);
      Body(parts.actors[4]).setPos(_pos.x - 3, _pos.y + 3);
      Body(parts.actors[5]).setPos(_pos.x + 3, _pos.y + 3);
      for (var i:int = 0; i < 6; i++)
        Body(parts.actors[i]).clearPosHistory();
      restartCnt = 0;
      fireCnt = 0;
      fireIdx = 0;
      fireDeg = 0;
      energy = MAX_ENERGY;
      energyWarningCnt = 0;
      noBounceCnt = 0;
    }

    public function setHidden():void {
      restartCnt = 999999;
    }

    private function setGameover():void {
      setHidden();
      gameoverCnt = 1;
      aPressed = true;
      mainLoop.startGameover();
    }

    public function update():void {
      Letter.drawNum(_height, field.size.x - 10, field.size.y - 22);
      Letter.drawString("m", field.size.x + 11, field.size.y - 17, 0.66);
      if (isTitle)
        return;
      handleExtendEnergy();
      for (var i:int = 0; i < left; i++)
        drawShape(20 + i * 30, field.size.y - 27);
      if (restartCnt > 0) {
        if (restartCnt < 10000) {
          restartCnt--;
          if (restartCnt == 0)
            respawn();
        }
        field.scrollVel += (0 - field.scrollVel) * 0.1;
        if (gameoverCnt > 0) {
          gameoverCnt++;
          if (gameoverCnt > 300) {
            mainLoop.startTitle();
            return;
          }
          if (gameoverCnt > 15) {
            Letter.drawString("GAME OVER", 120, 200, 0.7);
            if (keyState.a) {
              if (!aPressed)
                mainLoop.startTitle();
              aPressed = true;
            } else {
              aPressed = false;
            }
          }
        }
        return;
      }
      _pos.x = Body(parts.actors[1]).pos.x;
      _pos.y = Body(parts.actors[1]).pos.y;
      var svy:Number = field.upperLeftPos.y + field.size.y * 0.6 - pos.y;
      if (svy < MIN_SCROLL_VELOCITY_Y)
        svy = MIN_SCROLL_VELOCITY_Y;
      field.scrollVel += (svy * SCROLL_VELOCITY_Y_RATIO - field.scrollVel) * 0.1;
      var vx:Number = 0;
      var vy:Number = 0;
      if (keyState.left)
        vx = -1;
      if (keyState.right)
        vx = 1;
      if (keyState.up)
        vy = -1;
      if (keyState.down)
        vy = 1;
      if (vx != 0 && vy != 0) {
        vx *= 0.7;
        vy *= 0.7;
      }
      fireDeg += (-vx * Math.PI / 2 - fireDeg) * 0.1;
      vx *= MOVEMENT_FORCE;
      vy *= MOVEMENT_FORCE;
      if (vx < 0) {
        addForce(1, vx / 2, 0);
        addForce(3, vx / 2, 0);
      } else if (vx > 0) {
        addForce(3, vx, 0);
      }
      if (vy < 0) {
        addForce(1, 0, vy);
      } else if (vy > 0) {
        addForce(4, 0, vy / 2);
        addForce(5, 0, vy / 2);
      }
      fireCnt--;
      if (keyState.a) {
        if (!aPressed && fireCnt <= 0) {
          fireIdx++;
          fire((fireIdx % 2) + 2);
        }
      } else {
        aPressed = false;
      }
      Body(parts.actors[0]).addForce(0, -3);
      Body(parts.actors[2]).addForce(-1, -0.5);
      Body(parts.actors[3]).addForce(1, -0.5);
      Body(parts.actors[4]).addForce(-0.5, 2);
      Body(parts.actors[5]).addForce(0.5, 2);
      noBounceCnt--;
      if (_pos.y > field.lowerRightPos.y - field.size.y * 0.04) {
        _pos.y = field.lowerRightPos.y - field.size.y * 0.06;
        Body(parts.actors[1]).setPos(_pos.x, _pos.y);
        if (noBounceCnt <= 0) {
          Body(parts.actors[1]).addForce(0, -300);
          noBounceCnt = 10;
        }
      }
      cords.update();
      parts.update();
      var ch:int = (field.size.y - _pos.y) / Field.PIXEL_METER_RATIO;
      if (ch > _height)
        _height = ch;
      handleEnergy();
    }

    private function handleEnergy():void {
      energy -= stage.energyDecreaseAmount;
      if (energy <= 0) {
        destroy();
        return;
      }
      //Letter.drawNum(int(energy), field.size.x + 5, 26);
      rect.width = energy * 0.0333;
      rect.x = field.size.x + 18 - rect.width;
      rect.y = 2;
      if (energy > MAX_ENERGY / 5) {
        buffer.fillRect(rect, 0x00ff00);
      } else {
        buffer.fillRect(rect, 0xffff00);
        if (energyWarningCnt % 30 == 0)
          sound.playSe(6);
        energyWarningCnt++;
      }
    }

    private function handleExtendEnergy():void {
      rect.width = extendEnergy * 0.0333;
      rect.x = 12;
      rect.y = Lbr.SCREEN_HEIGHT - 7;
      buffer.fillRect(rect, 0x00ff00);
    }

    private function fire(idx:int):void {
      fireCnt = FIRE_INTERVAL;
      var f:Fragment = Fragment(fragments.getInstance());
      if (f == null)
        return;
      var sc:SoundChannel = sound.playSe(1);
      var pa:Number = -fireDeg;
      if (pa > 1)
        pa = 1;
      else if (pa < -1)
        pa = -1;
      sound.setVolumeAndPan(sc, 1, pa);
      var fd:Number = fireDeg + Math.PI;
      f.add(Body(parts.actors[idx]).pos.x + Math.sin(fd) * 20,
            Body(parts.actors[idx]).pos.y + Math.cos(fd) * 20,
            fd, FIRE_FORCE, 100, 100, 255);
      addForce(idx, -Math.sin(fd) * FIRE_RECOIL_FORCE, -Math.cos(fd) * FIRE_RECOIL_FORCE);
      for (var i:int = 0; i < 5; i++) {
        var pt:Particle = Particle(particles.getInstanceForced());
        pt.addVel(Body(parts.actors[idx]).pos.x,
                  Body(parts.actors[idx]).pos.y,
                  Math.sin(fd) * 10 + Math.random() * 10 - 5,
                  Math.cos(fd) * 10 + Math.random() * 10 - 5, 100, 100, 255);
      }
    }

    private function addForce(idx:int, vx:Number, vy:Number):void {
      var b:Body = Body(parts.actors[idx]);
      b.addForce(vx, vy);
    }

    public function drawShape(x:Number, y:Number):void {
      Cord(cords.actors[0]).drawShape(x, y, x, y - 10);
      Cord(cords.actors[1]).drawShape(x, y, x - 10, y - 2);
      Cord(cords.actors[2]).drawShape(x, y, x + 10, y - 2);
      Cord(cords.actors[3]).drawShape(x, y, x - 5, y + 15);
      Cord(cords.actors[4]).drawShape(x, y, x + 5, y + 15);
      shapePos.x = x;
      shapePos.y = y - 10;
      Body(parts.actors[0]).drawShape(shapePos);
      shapePos.x = x;
      shapePos.y = y;
      Body(parts.actors[1]).drawShape(shapePos);
      shapePos.x = x - 10;
      shapePos.y = y - 2;
      Body(parts.actors[2]).drawShape(shapePos);
      shapePos.x = x + 10;
      shapePos.y = y - 2;
      Body(parts.actors[3]).drawShape(shapePos);
      shapePos.x = x - 5;
      shapePos.y = y + 15;
      Body(parts.actors[4]).drawShape(shapePos);
      shapePos.x = x + 5;
      shapePos.y = y + 15;
      Body(parts.actors[5]).drawShape(shapePos);
    }

    private static const CONNECTED_IDX:Array = [0, 2, 3, 4, 5];

    public function drawBlur():void {
      var t:Number = 0.1;
      for (var i:int = Body.POS_HISTORY_MAX; i >= 1; i--) {
        var b1p:Vector = Body(parts.actors[1]).getHistoryPos(i);
        for (var j:int = 0; j < 5; j++)
          Cord(cords.actors[j]).drawShapeBlur(b1p, Body(parts.actors[CONNECTED_IDX[j]]).getHistoryPos(i), t);
        for (var k:int = 0; k < 6; k++)
          Body(parts.actors[k]).drawShapeBlur(Body(parts.actors[k]).getHistoryPos(i), t);
        t += 0.1;
      }
    }

    public function getEnergy():void {
      sound.playSe(5);
      addEnergy(MAX_ENERGY / 5);
    }

    public function getSmallEnergy():void {
      sound.playSe(7);
      addEnergy(MAX_ENERGY / 150);
    }

    private function addEnergy(e:Number):void {
      energy += e;
      if (energy > MAX_ENERGY)
        energy = MAX_ENERGY;
      if (left < 9)
        extendEnergy += e / ((1 + left) * (1 + left));
      if (extendEnergy > MAX_ENERGY) {
        extendEnergy = 0;
        extend();
      }
    }

    private function extend():void {
      sound.playSe(8);
      left++;
    }

    public function getSoundVolume(p:Vector):Number {
      return 1 / (Math.abs(p.y - _pos.y) * 0.005 + 1);
    }

    public function getSoundPan(p:Vector):Number {
      var pa:Number = (p.x - _pos.x) / field.size.x * 4;
      if (pa < -1)
        pa = -1;
      else if (pa > 1)
        pa = 1;
      return pa;
    }

    public function destroy():void {
      sound.playSe(4);
      left--;
      mainLoop.playerDestroyed();
      for (var i:int = 0; i < 256; i++) {
        var p:Particle = Particle(particles.getInstanceForced());
        if (Math.random() < 0.5)
          p.add(_pos.x, _pos.y, Math.random() * Math.PI * 2, 0.1 + Math.random() * 7, 0, 255, 0, 0);
        else
          p.add(_pos.x, _pos.y, Math.random() * Math.PI * 2, 0.1 + Math.random() * 7, 0, 255, 255, 0);
      }
      if (left >= 0)
        restartCnt = RESTART_CNT;
      else
        setGameover();
    }

    public function get pos():Vector {
      return _pos;
    }

    public function get hasCollision():Boolean {
      return (restartCnt <= 0);
    }

    public function get height():int {
      return _height;
    }
  }
}
