/*
 * $Id: SolidBall.as,v 1.1 2007/09/06 12:35:42 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import flash.media.SoundChannel;
  import flash.media.SoundTransform;
  import org.cove.ape.Group;
  import org.cove.ape.CircleParticle;
  import abagames.util.Vector;

  /**
   * Solid (undestroyable) ball.
   */
  public class SolidBall extends ApeParticleActor {
    private static const BALL_STRIP_WIDTH:Number = 3;
    private var buffer:BitmapData;
    private var field:Field;
    private var player:Player;
    private var fragments:FragmentPool;
    private var particles:ParticlePool;
    private var sound:SoundHandler;
    private var radius:Number;
    private var hardness:Number;
    private var burstRadius:Number;
    private var rect:Rectangle;
    private var cnt:int;

    public function SolidBall(buffer:BitmapData, field:Field, player:Player,
                            fragments:FragmentPool, particles:ParticlePool,
                            sound:SoundHandler, group:Group) {
      this.buffer = buffer;
      this.field = field;
      this.player = player;
      this.fragments = fragments;
      this.particles = particles;
      this.sound = sound;
      _particle = new CircleParticle(0, 0, 6);
      setGroup(group);
      rect = new Rectangle(0, 0, BALL_STRIP_WIDTH, BALL_STRIP_WIDTH);
    }

    public function add(x:Number, y:Number, radius:Number):void {
      cnt = 0;
      addToGroup();
      _particle.visible = false;
      this.radius = radius;
      CircleParticle(_particle).radius = radius;
      _particle.elasticity = 0.1;
      _particle.mass = radius * 0.15;
      _particle.friction = 0.01;
      setPos(x, y);
    }

    public override function update():void {
      super.update();
      if (!field.isInY(_pos.y)) {
        remove();
        return;
      }
      cnt++;
      if (cnt > 600) {
        destroy();
        return;
      }
      draw();
    }

    private function draw():void {
      var color:int;
      var d:int = (cnt - 500) / 20;
      if (d > 0 && (cnt % ((5 - d) * 2) == 0))
        color = 0xccaaaa;
      else
        color = 0xff4444;
      drawBox(_pos.x, _pos.y + field.scrollY, radius, color);
      for (var ox:Number = BALL_STRIP_WIDTH * 2; ox < radius; ox += (BALL_STRIP_WIDTH * 2)) {
        var w:Number = Math.sqrt(radius * radius - ox * ox);
        drawBox(_pos.x + ox, _pos.y + field.scrollY, w, color);
        drawBox(_pos.x - ox, _pos.y + field.scrollY, w, color);
      }
    }

    private function drawBox(x:Number, y:Number, w:Number, color:int):void {
      rect.x = x - rect.width / 2;
      rect.y = y - w;
      rect.height = w * 2;
      buffer.fillRect(rect, color);
    }

    public function drawShape(p:Vector):void {
      var r:Number = 20;
      var color:int = 0xff4444;
      drawBox(p.x, p.y, r, color);
      for (var ox:Number = BALL_STRIP_WIDTH * 2; ox < r; ox += (BALL_STRIP_WIDTH * 2)) {
        var w:Number = Math.sqrt(r * r - ox * ox);
        drawBox(p.x + ox, p.y, w, color);
        drawBox(p.x - ox, p.y, w, color);
      }
    }

    public function destroy(withSe:Boolean = true):void {
      var i:int;
      var d:Number;
      for (i = 0; i < radius * 3 * 0.4; i++) {
        var pt:Particle = Particle(particles.getInstanceForced());
        pt.add(_pos.x, _pos.y, Math.random() * Math.PI * 2, 5 + Math.random() * 15, 255, 100, 255);
      }
      if (withSe) {
        for (i = 0; i < radius * 3 * 0.2; i++) {
          var f:Fragment = Fragment(fragments.getInstance());
          if (f == null)
            break;
          d = Math.random() * Math.PI * 2;
          f.add(_pos.x + Math.sin(d) * radius, _pos.y + Math.cos(d) * radius,
                d, 100 * (1 + Math.random()), 255, 100, 255);
        }
        var sc:SoundChannel = sound.playSe(2);
        sound.setVolumeAndPan(sc, player.getSoundVolume(_pos), player.getSoundPan(_pos));
      }
      remove();
    }

    public override function remove():void {
      removeFromGroup();
      super.remove();
    }
  }
}
