/*
 * $Id: Title.as,v 1.2 2007/09/06 12:35:42 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import abagames.util.Vector;
  import abagames.util.KeyState;

  /**
   * Title screen.
   */
  public class Title {
    private var buffer:BitmapData;
    private var field:Field;
    private var keyState:KeyState;
    private var mainLoop:MainLoop;
    private var player:Player;
    private var balls:BallPool;
    private var solidBalls:SolidBallPool;
    private var mines:MinePool;
    private var energies:EnergyPool;
    private var aPressed:Boolean;
    private var rect:Rectangle;
    private var shapePos:Vector;
    private var cnt:int;
    private var highScores:Array;

    public function Title(buffer:BitmapData, field:Field, keyState:KeyState,
                        player:Player, store:Store,
                        balls:BallPool, solidBalls:SolidBallPool,
                        mines:MinePool, energies:EnergyPool, mainLoop:MainLoop) {
      this.buffer = buffer;
      this.field = field;
      this.keyState = keyState;
      this.player = player;
      this.balls = balls;
      this.solidBalls = solidBalls;
      this.mines = mines;
      this.energies = energies;
      this.mainLoop = mainLoop;
      highScores = store.highScores;
      rect = new Rectangle;
      shapePos = new Vector;
    }

    public function init():void {
      aPressed = true;
      cnt = 0;
    }

    public function update():void {
      cnt++;
      drawTitle();
      if ((cnt % 40) < 20)
        Letter.drawString("PUSH SHOT BUTTON", 80, 400, 0.7);
      if ((cnt % 640) < 320)
        drawHighScores(cnt % 640);
      else
        drawCharacters((cnt % 640) - 320);
      if (keyState.a) {
        if (!aPressed)
          mainLoop.startInGame();
        aPressed = true;
      } else {
        aPressed = false;
      }
    }

    private function drawTitle():void {
      Letter.drawString("LEAST", 120, 30, 0.9);
      Letter.drawString("BEST", 150, 70, 0.9);
      Letter.drawString("ROOM", 180, 110, 0.9);
      drawBox(105, 10, 1, 120);
      drawBox(247, 10, 1, 120);
      drawBox(105, 49, 106, 1);
      drawBox(141, 90, 106, 1);
    }

    private const ORDER_STRINGS:Array = ["ST", "ND", "RD", "TH"];

    private function drawHighScores(c:int):void {
      var rk:int = int(c / 15);
      var sz:Number = 0.65;
      if (rk > Store.HIGH_SCORES_NUM)
        rk = Store.HIGH_SCORES_NUM;
      for (var i:int = 0; i < rk; i++) {
        var y:int = i * 20 + 160;
        Letter.drawNum(i + 1, 105, y, sz);
        var osi:int = i;
        if (osi > 3)
          osi = 3;
        Letter.drawString(ORDER_STRINGS[osi], 120, y, sz);
        Letter.drawNum(highScores[i], 230, y, sz);
        Letter.drawString("m", 245, y + 2, sz * 0.8);
      }
    }

    private function drawCharacters(c:int):void {
      if (c < 30)
        return;
      player.drawShape(120, 180);
      Letter.drawString("YOU", 190, 180, 0.7);
      if (c < 60)
        return;
      shapePos.x = 95;
      shapePos.y = 230;
      Ball(balls.actors[0]).drawShape(shapePos);
      shapePos.x = 145;
      SolidBall(solidBalls.actors[0]).drawShape(shapePos);
      Letter.drawString("BALL", 190, 230, 0.7);
      if (c < 90)
        return;
      shapePos.x = 120;
      shapePos.y = 280;
      Mine(mines.actors[0]).drawShape(shapePos);
      Letter.drawString("MINE", 190, 280, 0.7);
      if (c < 120)
        return;
      shapePos.y = 330;
      Energy(energies.actors[0]).drawShape(shapePos);
      Letter.drawString("ENERGY", 190, 330, 0.7);
    }

    private function drawBox(x:int, y:int, w:int, h:int):void {
      rect.x = x;
      rect.y = y;
      rect.width = w;
      rect.height = h;
      buffer.fillRect(rect, 0xcccccc);
    }
  }
}
