/*
 * $Id: ActorPool.as,v 1.1.1.1 2007/08/19 01:14:45 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.util {
  import abagames.util.Actor;

  /**
   * An object pool for actors.
   */
  public class ActorPool {
    protected var _actors:Array;
    protected var _actorNum:int;
    protected var actorIdx:int;
    protected var _maxInstancesNum:int;
    protected var _instancesNum:int;

    public function ActorPool(n:int) {
      _actors = new Array;
      for (var i:int = 0; i < n; i++) {
        var a:Actor = createActor();
        a.setActorPool(this);
        a.index = i;
        a.removed = false;
        _actors.push(a);
      }
      _maxInstancesNum = _instancesNum = n;
      clear();
    }

    public function clear():void {
      var an:int = _actorNum;
      for (var i:int = 0; i < an; i++)
        _actors[0].remove();
      _actorNum = 0;
      actorIdx = -1;
    }

    public function getInstance():Actor {
      if (_actorNum >= _instancesNum)
        return null;
      _actorNum++;
      return _actors[_actorNum - 1];
    }

    public function get hasNextInstance():Boolean {
      return (_actorNum < _instancesNum);
    }

    public function getInstanceForced():Actor {
      var a:Actor = getInstance();
      if (a != null)
        return a;
      actorIdx++;
      if (actorIdx >= _actors.length)
        actorIdx = 0;
      return _actors[actorIdx];
    }

    public function remove(idx:int):void {
      _actorNum--;
      var ta:Actor = _actors[idx];
      _actors[idx] = _actors[_actorNum];
      _actors[idx].index = idx;
      _actors[_actorNum] = ta;
      _actors[_actorNum].index = _actorNum;
      _actors[_actorNum].removed = true;
    }

    public function update():void {
      for (var i:int = 0; i < _actorNum; i++) {
        var a:Actor = _actors[i];
        a.update();
        if (a.removed) {
          a.removed = false;
          i--;
        }
      }
    }

    protected function createActor():Actor {
      return null;
    }

    public function existsActor(idx:int):Boolean {
      return (idx < _actorNum);
    }

    public function get actorNum():int {
      return _actorNum;
    }

    public function get maxInstancesNum():int {
      return _maxInstancesNum;
    }

    public function get instancesNum():int {
      return _instancesNum;
    }

    public function set instancesNum(v:int):void {
      if (v > _maxInstancesNum)
        _instancesNum = _maxInstancesNum;
      else
        _instancesNum = v;
    }

    public function get actors():Array {
      return _actors;
    }
  }
}
